<?php defined('SYSPATH') || die('No direct script access.');

class Controller_Authentication extends Controller
{
    /** @var  User */
    private $user;

    /** @var Twig */
    private $twig;

    private $OpenIDAuthAllowed, $GradeAuthAllowed;

    public function before() {
        parent::before();

        $this->user = User::instance();
        $this->twig = Twig::factory();
        $this->OpenIDAuthAllowed = Model_System::loadConfig()->OpenIDAuthAllowed;
        $this->GradeAuthAllowed = Model_System::loadConfig()->GradeAuthAllowed;
    }

    public function after() {
        $this->twig->set([
            'Markdown' => new Parsedown(),
            'Blocks' => [
                'News'    => Model_System::getChangeLog('news.md'),
                'Updates' => Model_System::getChangeLog('updates.md'),
            ]
        ]);
        $this->response->body($this->twig);

        parent::after();
    }

    // for holy RS & dev group
    public function action_sign_anyway() {
        if ($this->user->isSignedIn())
            $this->redirect('/');
        $type = $this->request->param('type', $default = 'in');

        $this->twig->set([
            'OpenIDAuthAllowed' => $this->OpenIDAuthAllowed,
            'GradeAuthAllowed'  => $this->GradeAuthAllowed,
        ])->set_filename('sign/' . $type);
    }

    // for mere mortals
    public function action_sign() {
        $info = Model_Account::getMaintenanceInfo();

        if ($info['active']) {
            $this->response->status(503);
            $this->twig->set([
                'code'    => 503,
                'title'   => 'Закрыто на техобслуживание!',
                'message' => "Восстановление работы сервиса: " . $info['return'],
            ])->set_filename('errors/http');
            return;
        }

        $this->action_sign_anyway();
    }

    public function action_logout() {
        $this->user->signOut();
        $this->redirect('sign', 302);
    }

    public function action_remind() {
        if (!$this->GradeAuthAllowed)
            throw HTTP_Exception::factory(404);

        $this->twig->set_filename('sign/remindpass');
    }

    public function action_restore() {
        if (!$this->GradeAuthAllowed)
            throw HTTP_Exception::factory(404);

        $token = $this->request->param('token');

        if (!Account::checkToken($token)) {
            $message = "Данная ссылка для восстановления пароля более не действительна!\n" .
                "Либо истекло время действия ссылки, либо она уже была использована.";
            throw HTTP_Exception::factory(403, $message);
        }

        $this->twig->set('Token', $token)->set_filename('sign/changepass');
    }
}
